==> simplellmfunc-0.1.2.tar.gz: simplellmfunc-0.1.2/SimpleLLMFunc/interface/zhipu.py <==
import json
import time
from typing import Generator, Optional, Dict, List, Union, Literal, Iterable, Any
from openai import OpenAI
# 修复导入路径
from SimpleLLMFunc.interface.llm_interface import LLM_Interface
from SimpleLLMFunc.interface.key_pool import APIKeyPool
# 修复全局日志器函数导入
from SimpleLLMFunc.logger import app_log, push_warning, push_error, get_location, get_current_trace_id

# TODO:
# 计划后续迁移到ZhipuAI自己的SDK上以获得更好的支持

class Zhipu(LLM_Interface):
    def __init__(
        self,
        api_key_pool: APIKeyPool,
        model_name: Literal[
            "glm-4-flash",
            "glm-4-air",
            "glm-4-airx",
            "glm-4-long",
            "glm-4-plus",
            "glm-4-0520",
        ],
        max_retries: int = 5,  # 新增最大重试次数参数
        retry_delay: float = 1.0,  # 新增重试延迟时间
    ):
        super().__init__(api_key_pool, model_name)
        self.max_retries = max_retries
        self.retry_delay = retry_delay
        self.base_url = "https://open.bigmodel.cn/api/paas/v4/"

        self.model_list = [
            "glm-4-flash",
            "glm-4-air",
            "glm-4-airx",
            "glm-4-long",
            "glm-4-plus",
            "glm-4-0520",
        ]

        if model_name not in self.model_list:
            location = get_location()
            push_warning(
                f"model_name should be one of {self.model_list}", location=get_location()
            )
        self.model_name = model_name

        self.key_pool = api_key_pool
        self.client = OpenAI(api_key=api_key_pool.get_least_loaded_key(), base_url=self.base_url)

    def chat(
        self,
        trace_id: str = get_current_trace_id(),
        stream: Literal[False] = False,
        messages: Iterable[Dict[str, str]] = [{"role": "system", "content": "你是一位乐于助人的助手，可以帮助用户解决各种问题。"}],
        timeout: Optional[int] = 20,
        *args,
        **kwargs,
    ) -> Dict[Any, Any]:
        key = self.key_pool.get_least_loaded_key()
        self.client = OpenAI(api_key=key, base_url=self.base_url)
        
        attempt = 0
        while attempt < self.max_retries:
            try:

                self.key_pool.increment_task_count(key)
                data = json.dumps(messages, ensure_ascii=False, indent=4)
                #data = data[: min(512, len(data))]
                app_log(
                    f"Zhipu::chat: {self.model_name} request with API key: {key}, and message: {data}",
                    location=get_location()
                )
                response: Dict[Any, Any] = self.client.chat.completions.create(  # type: ignore
                    messages=messages,   # type: ignore
                    model=self.model_name,
                    stream=stream,
                    timeout=timeout,
                    *args,
                    **kwargs,
                )

                self.key_pool.decrement_task_count(key)
                return response  # 请求成功，返回结果
            except Exception as e:

                self.key_pool.decrement_task_count(key)
                attempt += 1
                location = get_location()
                data = json.dumps(messages, ensure_ascii=False, indent=4)
                #data = data[: min(512, len(data))]
                push_warning(
                    f"{self.model_name} Interface attempt {attempt} failed: With message : {data} send, \n but exception : {str(e)} was caught",
                    location=get_location(),
                )

                key = self.key_pool.get_least_loaded_key()
                self.client = OpenAI(api_key=key, base_url=self.base_url)

                if attempt >= self.max_retries:
                    push_error(
                        f"Max retries reached. {self.model_name} Failed to get a response for {data}",
                        location=location,
                    )
                    raise e  # 达到最大重试次数后抛出异常
                time.sleep(self.retry_delay)  # 重试前等待一段时间
        return {}  # 添加默认返回以满足类型检查，实际上这行代码永远不会执行

    def chat_stream(
        self,
        trace_id: str = get_current_trace_id(),
        stream: Literal[True] = True,
        messages: Iterable[Dict[str, str]] = [{"role": "system", "content": "你是一位乐于助人的助手，可以帮助用户解决各种问题。"}],
        timeout: Optional[int] = 10,
        *args,
        **kwargs,
    ) -> Generator[Dict[Any, Any], None, None]:
        key = self.key_pool.get_least_loaded_key()
        self.client = OpenAI(api_key=key, base_url=self.base_url)

        attempt = 0
        while attempt < self.max_retries:
            try:
                self.key_pool.increment_task_count(key)
                data = json.dumps(messages, ensure_ascii=False, indent=4)
                #data = data[: min(512, len(data))]
                app_log(
                    f"Zhipu::chat_stream: {self.model_name} request with API key: {key}, and message: {data}",
                    location=get_location()
                )
                response: Generator[Dict[Any, Any], None, None] = self.client.chat.completions.create(  # type: ignore
                    messages=messages,  # type: ignore
                    model=self.model_name,
                    stream=stream,
                    timeout=timeout,
                    *args,
                    **kwargs,
                )

                for chunk in response:
                    yield chunk  # 按块返回生成器中的数据

                self.key_pool.decrement_task_count(key)
                break  # 如果成功，跳出重试循环
            except Exception as e:

                self.key_pool.decrement_task_count(key)
                attempt += 1
                location = get_location()
                data = json.dumps(messages, ensure_ascii=False, indent=4)
                #data = data[: min(512, len(data))]
                push_warning(
                    f"{self.model_name} Interface attempt {attempt} failed: With message : {data} send, \n but exception : {str(e)} was caught",
                    location=get_location()
                )

                key = self.key_pool.get_least_loaded_key()
                self.client = OpenAI(api_key=key, base_url=self.base_url)

                if attempt >= self.max_retries:
                    push_error(
                        f"Max retries reached. {self.model_name} Failed to get a response for {data}",
                        location=get_location()
                    )
                    raise e
                time.sleep(self.retry_delay)
        
        # 下面是一个空生成器，用于满足类型检查，实际上永远不会执行到这里
        if False:
            yield {}


if __name__ == "__main__":

    # 测试interface
    
    from SimpleLLMFunc.interface.key_pool import APIKeyPool
    from typing import List
    from SimpleLLMFunc.config import global_settings
    import re
    
    # 修改后的正则表达式模式，保留减号
    pattern = re.compile(r'[\s\n]+')
    
    # 去除多余字符的函数
    def clean_api_keys(api_key_list: List[str]) -> List[str]:
        return [pattern.sub('', key.strip()) for key in api_key_list]
    
    # 直接使用 global_config 中的 API KEY 列表，不需要 split
    app_log(
        f"ZHIPUAI_API_KEY_LIST: {global_settings.ZHIPU_API_KEYS}",
        trace_id="test_trace_id"
    )
    
    ZHIPUAI_API_KEY_POOL = APIKeyPool(global_settings.ZHIPU_API_KEYS, "zhipu")
    
    ZhipuAI_glm_4_flash_Interface = Zhipu(ZHIPUAI_API_KEY_POOL, "glm-4-flash")
    
    # 测试 chat 方法
    trace_id = "test_trace_id"
    messages = [{"role": "user", "content": "你好"}]
    response = ZhipuAI_glm_4_flash_Interface.chat(trace_id, messages=messages)
    print("Chat response:", response)
    
    # 测试 chat_stream 方法
    trace_id = "test_trace_id"
    messages = [{"role": "user", "content": "你好"}]
    response = ZhipuAI_glm_4_flash_Interface.chat_stream(trace_id, messages=messages)
    print("Chat stream response:")
    for chunk in response:
        print(chunk)

==> simplellmfunc-0.1.4.tar.gz: simplellmfunc-0.1.4/SimpleLLMFunc/interface/zhipu.py <==
import json
import time
from typing import Generator, Optional, Dict, List, Union, Literal, Iterable, Any

# 修改导入路径
from SimpleLLMFunc.interface.openai_compatible import OpenAICompatible
from SimpleLLMFunc.interface.key_pool import APIKeyPool
# 修复全局日志器函数导入
from SimpleLLMFunc.logger import app_log, push_warning, get_location, get_current_trace_id

# 定义智谱AI的基础URL
ZHIPU_BASE_URL = "https://open.bigmodel.cn/api/paas/v4/"

# 修改Zhipu类继承自OpenAICompatible
class Zhipu(OpenAICompatible):
    """智谱AI的LLM接口实现，基于OpenAICompatible"""

    def __init__(
        self,
        api_key_pool: APIKeyPool,
        model_name: Literal[
            "glm-4-flash",
            "glm-4-air",
            "glm-4-airx",
            "glm-4-long",
            "glm-4-plus",
            "glm-4-0520",
        ],
        max_retries: int = 5,
        retry_delay: float = 1.0,
    ):
        """初始化智谱AI接口

        Args:
            api_key_pool: API密钥池，用于管理和分配API密钥
            model_name: 要使用的智谱模型名称
            max_retries: 最大重试次数
            retry_delay: 重试间隔时间（秒）
        """
        # 智谱AI支持的模型列表
        allowed_models = [
            "glm-4-flash",
            "glm-4-air",
            "glm-4-airx",
            "glm-4-long",
            "glm-4-plus",
            "glm-4-0520",
        ]
        
        # 调用OpenAICompatible的初始化方法，传入智谱AI特定的参数
        super().__init__(
            api_key_pool=api_key_pool,
            model_name=model_name,
            base_url=ZHIPU_BASE_URL,
            max_retries=max_retries,
            retry_delay=retry_delay,
            allowed_models=allowed_models
        )


if __name__ == "__main__":
    # 测试interface
    
    from SimpleLLMFunc.interface.key_pool import APIKeyPool
    from typing import List
    from SimpleLLMFunc.config import global_settings
    import re
    
    # 修改后的正则表达式模式，保留减号
    pattern = re.compile(r'[\s\n]+')
    
    # 去除多余字符的函数
    def clean_api_keys(api_key_list: List[str]) -> List[str]:
        return [pattern.sub('', key.strip()) for key in api_key_list]
    
    # 直接使用 global_config 中的 API KEY 列表，不需要 split
    app_log(
        f"ZHIPUAI_API_KEY_LIST: {global_settings.ZHIPU_API_KEYS}",
        trace_id="test_trace_id"
    )
    
    ZHIPUAI_API_KEY_POOL = APIKeyPool(global_settings.ZHIPU_API_KEYS, "zhipu")
    
    ZhipuAI_glm_4_flash_Interface = Zhipu(ZHIPUAI_API_KEY_POOL, "glm-4-flash")
    
    # 测试 chat 方法
    trace_id = "test_trace_id"
    messages = [{"role": "user", "content": "你好"}]
    response = ZhipuAI_glm_4_flash_Interface.chat(trace_id, messages=messages)
    print("Chat response:", response)
    
    # 测试 chat_stream 方法
    trace_id = "test_trace_id"
    messages = [{"role": "user", "content": "你好"}]
    response = ZhipuAI_glm_4_flash_Interface.chat_stream(trace_id, messages=messages)
    print("Chat stream response:")
    for chunk in response:
        print(chunk)
